import * as React from 'react';
import * as ReactDOM from 'react-dom';
import jsxRuntime from 'react/jsx-runtime';
import ts from 'typescript';

const normalizePath = (path, baseFileName)=>{
    if (path.startsWith('/')) {
        return path;
    }
    if (path.startsWith('.')) {
        const currentPath = baseFileName.split('/');
        currentPath.pop();
        while(1){
            if (path.startsWith('../')) {
                currentPath.pop();
                path = path.substring(3);
            } else if (path.startsWith('./')) {
                path = path.substring(2);
            } else {
                break;
            }
        }
        currentPath.push(path);
        return currentPath.join('/');
    }
    return path;
};

const mergeResolve = (resolve = {})=>{
    const mergedResolve = {
        ...resolve,
        extensions: resolve.extensions || [
            '.js'
        ],
        externals: {
            ...resolve.externals
        },
        cdnPrefix: 'https://unpkg.com'
    };
    return mergedResolve;
};
const createClosureMap = ()=>{
    const closureMap = {
        'react': ()=>React,
        'react-dom': ()=>ReactDOM,
        'react/jsx-runtime': ()=>jsxRuntime
    };
    return closureMap;
};
const codeToClosure = (code, fileName)=>{
    // Is there a better way to do this?
    // eslint-disable-next-line no-eval
    return eval(`(_require) => {
    const require = (path) => _require(path, '${fileName}');
    const module = {};
    const exports = {};
    ${code}
    return module.exports ?? exports;
  }`);
};
const createRequireFn = (closureMap, requireFn, resolve)=>{
    const fn = (path, currentFileName)=>{
        path = normalizePath(path, currentFileName);
        if (requireFn) {
            return requireFn(path, currentFileName);
        }
        const tried = [
            path
        ];
        let closure = closureMap[path];
        for (const extension of resolve.extensions){
            if (closure) {
                break;
            }
            const pathWithExtension = `${path}${extension}`;
            closure = closureMap[pathWithExtension];
            tried.push(pathWithExtension);
        }
        for (const extension of resolve.extensions){
            if (closure) {
                break;
            }
            const pathWithExtension = `${path}/index${extension}`;
            closure = closureMap[pathWithExtension];
            tried.push(pathWithExtension);
        }
        if (!closure) {
            throw new Error(`${currentFileName}: Cannot find module '${path}' (tried ${tried.join(', ')})`);
        }
        return closure(fn);
    };
    return fn;
};

const loadUMDModule = async (url)=>{
    const res = await fetch(url);
    const code = await res.text();
    return codeToClosure(code, url);
};
const loadExternalsToClosureMap = async (resolve, closureMap)=>{
    // TODO: parallelise
    for (const [name, value] of Object.entries(resolve.externals)){
        if (/^https?:\/\//.test(value)) {
            closureMap[name] = await loadUMDModule(value);
        } else if (window[value]) {
            closureMap[name] = ()=>window[value];
        } else {
            // CDN will read the "brower" field in `package.json`
            // and return the UMD bundle
            closureMap[name] = await loadUMDModule(`${resolve.cdnPrefix}/${name}@${value}`);
        }
    }
};

function _extends() {
    _extends = Object.assign || function(target) {
        for(var i = 1; i < arguments.length; i++){
            var source = arguments[i];
            for(var key in source){
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key];
                }
            }
        }
        return target;
    };
    return _extends.apply(this, arguments);
}
var hasLocalStorage = false;
try {
    hasLocalStorage = typeof localStorage !== "undefined";
} catch (error) {}
var hasProcess = typeof process !== "undefined";
var shouldDebug = hasLocalStorage && /*#__PURE__*/ localStorage.getItem("DEBUG") || hasProcess && process.env.DEBUG;
var debugLog = shouldDebug ? console.log : function(_message) {
    return "";
};
/**
 * Makes a virtual copy of the TypeScript environment. This is the main API you want to be using with
 * @typescript/vfs. A lot of the other exposed functions are used by this function to get set up.
 *
 * @param sys an object which conforms to the TS Sys (a shim over read/write access to the fs)
 * @param rootFiles a list of files which are considered inside the project
 * @param ts a copy pf the TypeScript module
 * @param compilerOptions the options for this compiler run
 * @param customTransformers custom transformers for this compiler run
 */ function createVirtualTypeScriptEnvironment(sys, rootFiles, ts, compilerOptions, customTransformers) {
    if (compilerOptions === void 0) {
        compilerOptions = {};
    }
    var mergedCompilerOpts = _extends({}, defaultCompilerOptions(ts), compilerOptions);
    var _createVirtualLanguag = createVirtualLanguageServiceHost(sys, rootFiles, mergedCompilerOpts, ts, customTransformers), languageServiceHost = _createVirtualLanguag.languageServiceHost, _updateFile = _createVirtualLanguag.updateFile;
    var languageService = ts.createLanguageService(languageServiceHost);
    var diagnostics = languageService.getCompilerOptionsDiagnostics();
    if (diagnostics.length) {
        var compilerHost = createVirtualCompilerHost(sys, compilerOptions, ts);
        throw new Error(ts.formatDiagnostics(diagnostics, compilerHost.compilerHost));
    }
    return {
        // @ts-ignore
        name: "vfs",
        sys: sys,
        languageService: languageService,
        getSourceFile: function getSourceFile(fileName) {
            var _languageService$getP;
            return (_languageService$getP = languageService.getProgram()) == null ? void 0 : _languageService$getP.getSourceFile(fileName);
        },
        createFile: function createFile(fileName, content) {
            _updateFile(ts.createSourceFile(fileName, content, mergedCompilerOpts.target, false));
        },
        updateFile: function updateFile(fileName, content, optPrevTextSpan) {
            var prevSourceFile = languageService.getProgram().getSourceFile(fileName);
            if (!prevSourceFile) {
                throw new Error("Did not find a source file for " + fileName);
            }
            var prevFullContents = prevSourceFile.text; // TODO: Validate if the default text span has a fencepost error?
            var prevTextSpan = optPrevTextSpan != null ? optPrevTextSpan : ts.createTextSpan(0, prevFullContents.length);
            var newText = prevFullContents.slice(0, prevTextSpan.start) + content + prevFullContents.slice(prevTextSpan.start + prevTextSpan.length);
            var newSourceFile = ts.updateSourceFile(prevSourceFile, newText, {
                span: prevTextSpan,
                newLength: content.length
            });
            _updateFile(newSourceFile);
        }
    };
} // TODO: This could be replaced by grabbing: https://github.com/microsoft/TypeScript/blob/main/src/lib/libs.json
// and then using that to generate the list of files from the server, but it is not included in the npm package
/**
 * Grab the list of lib files for a particular target, will return a bit more than necessary (by including
 * the dom) but that's OK, we're really working with the constraint that you can't get a list of files
 * when running in a browser.
 *
 * @param target The compiler settings target baseline
 * @param ts A copy of the TypeScript module
 */ var knownLibFilesForCompilerOptions = function knownLibFilesForCompilerOptions(compilerOptions, ts) {
    var target = compilerOptions.target || ts.ScriptTarget.ES5;
    var lib = compilerOptions.lib || []; // Note that this will include files which can't be found for particular versions of TS
    // TODO: Replace this with some sort of API call if https://github.com/microsoft/TypeScript/pull/54011
    // or similar is merged.
    var files = [
        "lib.d.ts",
        "lib.decorators.d.ts",
        "lib.decorators.legacy.d.ts",
        "lib.dom.d.ts",
        "lib.dom.iterable.d.ts",
        "lib.webworker.d.ts",
        "lib.webworker.importscripts.d.ts",
        "lib.webworker.iterable.d.ts",
        "lib.scripthost.d.ts",
        "lib.es5.d.ts",
        "lib.es6.d.ts",
        "lib.es2015.collection.d.ts",
        "lib.es2015.core.d.ts",
        "lib.es2015.d.ts",
        "lib.es2015.generator.d.ts",
        "lib.es2015.iterable.d.ts",
        "lib.es2015.promise.d.ts",
        "lib.es2015.proxy.d.ts",
        "lib.es2015.reflect.d.ts",
        "lib.es2015.symbol.d.ts",
        "lib.es2015.symbol.wellknown.d.ts",
        "lib.es2016.array.include.d.ts",
        "lib.es2016.d.ts",
        "lib.es2016.full.d.ts",
        "lib.es2017.d.ts",
        "lib.es2017.date.d.ts",
        "lib.es2017.full.d.ts",
        "lib.es2017.intl.d.ts",
        "lib.es2017.object.d.ts",
        "lib.es2017.sharedmemory.d.ts",
        "lib.es2017.string.d.ts",
        "lib.es2017.typedarrays.d.ts",
        "lib.es2018.asyncgenerator.d.ts",
        "lib.es2018.asynciterable.d.ts",
        "lib.es2018.d.ts",
        "lib.es2018.full.d.ts",
        "lib.es2018.intl.d.ts",
        "lib.es2018.promise.d.ts",
        "lib.es2018.regexp.d.ts",
        "lib.es2019.array.d.ts",
        "lib.es2019.d.ts",
        "lib.es2019.full.d.ts",
        "lib.es2019.intl.d.ts",
        "lib.es2019.object.d.ts",
        "lib.es2019.string.d.ts",
        "lib.es2019.symbol.d.ts",
        "lib.es2020.bigint.d.ts",
        "lib.es2020.d.ts",
        "lib.es2020.date.d.ts",
        "lib.es2020.full.d.ts",
        "lib.es2020.intl.d.ts",
        "lib.es2020.number.d.ts",
        "lib.es2020.promise.d.ts",
        "lib.es2020.sharedmemory.d.ts",
        "lib.es2020.string.d.ts",
        "lib.es2020.symbol.wellknown.d.ts",
        "lib.es2021.d.ts",
        "lib.es2021.full.d.ts",
        "lib.es2021.intl.d.ts",
        "lib.es2021.promise.d.ts",
        "lib.es2021.string.d.ts",
        "lib.es2021.weakref.d.ts",
        "lib.es2022.array.d.ts",
        "lib.es2022.d.ts",
        "lib.es2022.error.d.ts",
        "lib.es2022.full.d.ts",
        "lib.es2022.intl.d.ts",
        "lib.es2022.object.d.ts",
        "lib.es2022.regexp.d.ts",
        "lib.es2022.sharedmemory.d.ts",
        "lib.es2022.string.d.ts",
        "lib.es2023.array.d.ts",
        "lib.es2023.collection.d.ts",
        "lib.es2023.d.ts",
        "lib.es2023.full.d.ts",
        "lib.esnext.array.d.ts",
        "lib.esnext.asynciterable.d.ts",
        "lib.esnext.bigint.d.ts",
        "lib.esnext.d.ts",
        "lib.esnext.decorators.d.ts",
        "lib.esnext.disposable.d.ts",
        "lib.esnext.full.d.ts",
        "lib.esnext.intl.d.ts",
        "lib.esnext.promise.d.ts",
        "lib.esnext.string.d.ts",
        "lib.esnext.symbol.d.ts",
        "lib.esnext.weakref.d.ts"
    ];
    var targetToCut = ts.ScriptTarget[target];
    var matches = files.filter(function(f) {
        return f.startsWith("lib." + targetToCut.toLowerCase());
    });
    var targetCutIndex = files.indexOf(matches.pop());
    var getMax = function getMax(array) {
        return array && array.length ? array.reduce(function(max, current) {
            return current > max ? current : max;
        }) : undefined;
    }; // Find the index for everything in
    var indexesForCutting = lib.map(function(lib) {
        var matches = files.filter(function(f) {
            return f.startsWith("lib." + lib.toLowerCase());
        });
        if (matches.length === 0) return 0;
        var cutIndex = files.indexOf(matches.pop());
        return cutIndex;
    });
    var libCutIndex = getMax(indexesForCutting) || 0;
    var finalCutIndex = Math.max(targetCutIndex, libCutIndex);
    return files.slice(0, finalCutIndex + 1);
};
/**
 * Create a virtual FS Map with the lib files from a particular TypeScript
 * version based on the target, Always includes dom ATM.
 *
 * @param options The compiler target, which dictates the libs to set up
 * @param version the versions of TypeScript which are supported
 * @param cache should the values be stored in local storage
 * @param ts a copy of the typescript import
 * @param lzstring an optional copy of the lz-string import
 * @param fetcher an optional replacement for the global fetch function (tests mainly)
 * @param storer an optional replacement for the localStorage global (tests mainly)
 */ var createDefaultMapFromCDN = function createDefaultMapFromCDN(options, version, cache, ts, lzstring, fetcher, storer) {
    var fetchlike = fetcher || fetch;
    var fsMap = new Map();
    var files = knownLibFilesForCompilerOptions(options, ts);
    var prefix = "https://typescript.azureedge.net/cdn/" + version + "/typescript/lib/";
    function zip(str) {
        return lzstring ? lzstring.compressToUTF16(str) : str;
    }
    function unzip(str) {
        return lzstring ? lzstring.decompressFromUTF16(str) : str;
    } // Map the known libs to a node fetch promise, then return the contents
    function uncached() {
        return Promise.all(files.map(function(lib) {
            return fetchlike(prefix + lib).then(function(resp) {
                return resp.text();
            });
        })).then(function(contents) {
            contents.forEach(function(text, index) {
                return fsMap.set("/" + files[index], text);
            });
        }) // Return a NOOP for .d.ts files which aren't in the current build of TypeScript
        ["catch"](function() {});
    } // A localstorage and lzzip aware version of the lib files
    function cached() {
        var storelike = storer || localStorage;
        var keys = Object.keys(storelike);
        keys.forEach(function(key) {
            // Remove anything which isn't from this version
            if (key.startsWith("ts-lib-") && !key.startsWith("ts-lib-" + version)) {
                storelike.removeItem(key);
            }
        });
        return Promise.all(files.map(function(lib) {
            var cacheKey = "ts-lib-" + version + "-" + lib;
            var content = storelike.getItem(cacheKey);
            if (!content) {
                // Make the API call and store the text concent in the cache
                return fetchlike(prefix + lib).then(function(resp) {
                    return resp.text();
                }).then(function(t) {
                    storelike.setItem(cacheKey, zip(t));
                    return t;
                }) // Return a NOOP for .d.ts files which aren't in the current build of TypeScript
                ["catch"](function() {});
            } else {
                return Promise.resolve(unzip(content));
            }
        })).then(function(contents) {
            contents.forEach(function(text, index) {
                if (text) {
                    var name = "/" + files[index];
                    fsMap.set(name, text);
                }
            });
        });
    }
    var func = cache ? cached : uncached;
    return func().then(function() {
        return fsMap;
    });
};
function notImplemented(methodName) {
    throw new Error("Method '" + methodName + "' is not implemented.");
}
function audit(name, fn) {
    return function() {
        for(var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++){
            args[_key] = arguments[_key];
        }
        var res = fn.apply(void 0, args);
        var smallres = typeof res === "string" ? res.slice(0, 80) + "..." : res;
        debugLog.apply(void 0, [
            "> " + name
        ].concat(args));
        debugLog("< " + smallres);
        return res;
    };
}
/** The default compiler options if TypeScript could ever change the compiler options */ var defaultCompilerOptions = function defaultCompilerOptions(ts) {
    return _extends({}, ts.getDefaultCompilerOptions(), {
        jsx: ts.JsxEmit.React,
        strict: true,
        esModuleInterop: true,
        module: ts.ModuleKind.ESNext,
        suppressOutputPathCheck: true,
        skipLibCheck: true,
        skipDefaultLibCheck: true,
        moduleResolution: ts.ModuleResolutionKind.NodeJs
    });
}; // "/DOM.d.ts" => "/lib.dom.d.ts"
var libize = function libize(path) {
    return path.replace("/", "/lib.").toLowerCase();
};
/**
 * Creates an in-memory System object which can be used in a TypeScript program, this
 * is what provides read/write aspects of the virtual fs
 */ function createSystem(files) {
    return {
        args: [],
        createDirectory: function createDirectory() {
            return notImplemented("createDirectory");
        },
        // TODO: could make a real file tree
        directoryExists: audit("directoryExists", function(directory) {
            return Array.from(files.keys()).some(function(path) {
                return path.startsWith(directory);
            });
        }),
        exit: function exit() {
            return notImplemented("exit");
        },
        fileExists: audit("fileExists", function(fileName) {
            return files.has(fileName) || files.has(libize(fileName));
        }),
        getCurrentDirectory: function getCurrentDirectory() {
            return "/";
        },
        getDirectories: function getDirectories() {
            return [];
        },
        getExecutingFilePath: function getExecutingFilePath() {
            return notImplemented("getExecutingFilePath");
        },
        readDirectory: audit("readDirectory", function(directory) {
            return directory === "/" ? Array.from(files.keys()) : [];
        }),
        readFile: audit("readFile", function(fileName) {
            return files.get(fileName) || files.get(libize(fileName));
        }),
        resolvePath: function resolvePath(path) {
            return path;
        },
        newLine: "\n",
        useCaseSensitiveFileNames: true,
        write: function write() {
            return notImplemented("write");
        },
        writeFile: function writeFile(fileName, contents) {
            files.set(fileName, contents);
        }
    };
}
/**
 * Creates an in-memory CompilerHost -which is essentially an extra wrapper to System
 * which works with TypeScript objects - returns both a compiler host, and a way to add new SourceFile
 * instances to the in-memory file system.
 */ function createVirtualCompilerHost(sys, compilerOptions, ts) {
    var sourceFiles = new Map();
    var save = function save(sourceFile) {
        sourceFiles.set(sourceFile.fileName, sourceFile);
        return sourceFile;
    };
    var vHost = {
        compilerHost: _extends({}, sys, {
            getCanonicalFileName: function getCanonicalFileName(fileName) {
                return fileName;
            },
            getDefaultLibFileName: function getDefaultLibFileName() {
                return "/" + ts.getDefaultLibFileName(compilerOptions);
            },
            // getDefaultLibLocation: () => '/',
            getDirectories: function getDirectories() {
                return [];
            },
            getNewLine: function getNewLine() {
                return sys.newLine;
            },
            getSourceFile: function getSourceFile(fileName) {
                return sourceFiles.get(fileName) || save(ts.createSourceFile(fileName, sys.readFile(fileName), compilerOptions.target || defaultCompilerOptions(ts).target, false));
            },
            useCaseSensitiveFileNames: function useCaseSensitiveFileNames() {
                return sys.useCaseSensitiveFileNames;
            }
        }),
        updateFile: function updateFile(sourceFile) {
            var alreadyExists = sourceFiles.has(sourceFile.fileName);
            sys.writeFile(sourceFile.fileName, sourceFile.text);
            sourceFiles.set(sourceFile.fileName, sourceFile);
            return alreadyExists;
        }
    };
    return vHost;
}
/**
 * Creates an object which can host a language service against the virtual file-system
 */ function createVirtualLanguageServiceHost(sys, rootFiles, compilerOptions, ts, customTransformers) {
    var fileNames = [].concat(rootFiles);
    var _createVirtualCompile = createVirtualCompilerHost(sys, compilerOptions, ts), compilerHost = _createVirtualCompile.compilerHost, _updateFile2 = _createVirtualCompile.updateFile;
    var fileVersions = new Map();
    var projectVersion = 0;
    var languageServiceHost = _extends({}, compilerHost, {
        getProjectVersion: function getProjectVersion() {
            return projectVersion.toString();
        },
        getCompilationSettings: function getCompilationSettings() {
            return compilerOptions;
        },
        getCustomTransformers: function getCustomTransformers() {
            return customTransformers;
        },
        // A couple weeks of 4.8 TypeScript nightlies had a bug where the Program's
        // list of files was just a reference to the array returned by this host method,
        // which means mutations by the host that ought to result in a new Program being
        // created were not detected, since the old list of files and the new list of files
        // were in fact a reference to the same underlying array. That was fixed in
        // https://github.com/microsoft/TypeScript/pull/49813, but since the twoslash runner
        // is used in bisecting for changes, it needs to guard against being busted in that
        // couple-week period, so we defensively make a slice here.
        getScriptFileNames: function getScriptFileNames() {
            return fileNames.slice();
        },
        getScriptSnapshot: function getScriptSnapshot(fileName) {
            var contents = sys.readFile(fileName);
            if (contents) {
                return ts.ScriptSnapshot.fromString(contents);
            }
            return;
        },
        getScriptVersion: function getScriptVersion(fileName) {
            return fileVersions.get(fileName) || "0";
        },
        writeFile: sys.writeFile
    });
    var lsHost = {
        languageServiceHost: languageServiceHost,
        updateFile: function updateFile(sourceFile) {
            projectVersion++;
            fileVersions.set(sourceFile.fileName, projectVersion.toString());
            if (!fileNames.includes(sourceFile.fileName)) {
                fileNames.push(sourceFile.fileName);
            }
            _updateFile2(sourceFile);
        }
    };
    return lsHost;
}

const compilerOptions = {
    allowArbitraryExtensions: true,
    allowNonTsExtensions: true,
    allowSyntheticDefaultImports: true,
    emitHelpers: false,
    esModuleInterop: true,
    jsx: ts.JsxEmit.ReactJSX,
    module: ts.ModuleKind.CommonJS,
    noEmitOnError: false,
    outDir: '/',
    resolveJsonModule: true,
    skipLibCheck: true,
    skipDefaultLibCheck: true,
    target: ts.ScriptTarget.ES2020
};
const createFsMap = async (sources)=>{
    const fsMap = await createDefaultMapFromCDN({
        target: compilerOptions.target
    }, ts.version, true, ts);
    for(const filename in sources){
        fsMap.set(filename === '_' ? 'index.tsx' : filename, sources[filename]);
    }
    return fsMap;
};
const createTsEnv = (fsMap)=>{
    const files = [
        ...fsMap.keys()
    ];
    const system = createSystem(fsMap);
    return createVirtualTypeScriptEnvironment(system, files, ts, compilerOptions);
};

const cssRe = /\.(?:c|le|sa|sc)ss|stylus$/i;
const jsRe = /\.m?[jt]sx?$/i;
const jsonRe = /\.json$/i;
const enforceOrderMap = {
    pre: 1,
    default: 0,
    post: -1
};
const normalizeRules = (rules)=>{
    const normalizedRules = rules.map((rule, index)=>({
            test: rule.test instanceof RegExp ? rule.test : new RegExp(rule.test),
            use: rule.use.map((loader)=>{
                if (typeof loader === 'function') {
                    return {
                        loader,
                        options: {}
                    };
                }
                return {
                    ...loader,
                    options: loader.options || {}
                };
            }),
            enforce: rule.enforce || 'default',
            _originalOrder: index
        }));
    normalizedRules.sort((a, b)=>{
        if (a.enforce === b.enforce) {
            return a._originalOrder - b._originalOrder;
        }
        return enforceOrderMap[a.enforce] - enforceOrderMap[b.enforce];
    });
    return normalizedRules;
};
const applyRules = (rules, filename, content)=>{
    let result = content;
    let error = null;
    const currentMeta = {
        filename,
        options: {}
    };
    for (const rule of normalizeRules(rules)){
        if (rule.test.test(filename)) {
            // forward: pitch
            let pitchLoaderIndex = 0;
            for(; pitchLoaderIndex < rule.use.length; pitchLoaderIndex++){
                const { loader, options } = rule.use[pitchLoaderIndex];
                currentMeta.options = options;
                if (loader.pitch) {
                    try {
                        const newContent = loader.pitch(result, currentMeta);
                        if (typeof newContent !== 'undefined') {
                            result = newContent;
                            break;
                        }
                    } catch (e) {
                        return {
                            content: result,
                            error: e
                        };
                    }
                }
            }
            // backward: normal
            for(let i = Math.min(pitchLoaderIndex, rule.use.length - 1); i >= 0; i--){
                const { loader, options } = rule.use[i];
                currentMeta.options = options;
                loader(result, currentMeta, (err, newContent)=>{
                    if (err) {
                        error = err;
                        return;
                    }
                    result = newContent;
                });
                if (error) {
                    return {
                        content: result,
                        error
                    };
                }
            }
        }
    }
    return {
        content: result,
        error: null
    };
};
const parseSources = (sources, rules, errors)=>{
    const cleanupFiles = [];
    const parsedSources = {};
    for (const [_filename, _content] of Object.entries(sources)){
        const filename = normalizePath(_filename.startsWith('/') ? _filename : `/${_filename}`, '/index.js');
        const { content, error } = applyRules(rules, filename, _content);
        if (error) {
            error.message = `${filename}: ${error.message}`;
            errors.push(error);
            continue;
        }
        if (jsRe.test(filename)) {
            parsedSources[filename] = content;
            continue;
        }
        if (jsonRe.test(filename)) {
            parsedSources[`${filename}.js`] = `export default ${content};`;
        } else if (cssRe.test(filename)) {
            parsedSources[filename] = `
const s = \`${content.trim()}\`;
let el = document.head.querySelector('style[data-tsx-browser-compiler-filename="${filename}"]');
if (!el) {
  el = document.createElement('style');
  el.setAttribute('data-tsx-browser-compiler-filename', '${filename}');
  document.head.appendChild(el);
}
el.textContent = s;
      `;
            cleanupFiles.push(filename);
        } else {
            errors.push(new Error(`${filename}: you may need a custom rule for this file type.`));
            parsedSources[filename] = content;
        }
    }
    return {
        parsedSources,
        cleanupFiles
    };
};

const VERSION = "1.0.0-alpha.1";
const errNoCode = new Error('No code emitted.');
const ignoredCode = [
    2307,
    6054,
    7016,
    7026
];
const noop = ()=>{};
const asyncTsxToElement = async ({ sources, entryFile = '/index.js', resolve, requireFn, rules = [], displayName = 'TsxToElement' })=>{
    const codeMap = [];
    const closureMap = createClosureMap();
    const errors = [];
    const { parsedSources, cleanupFiles } = parseSources(sources, rules, errors);
    const fsMap = await createFsMap(parsedSources);
    const env = createTsEnv(fsMap);
    for (const filename of fsMap.keys()){
        const emitOutput = env.languageService.getEmitOutput(filename);
        if (!emitOutput) {
            continue;
        }
        const diagnostics = env.languageService.getSemanticDiagnostics(filename);
        for (const diagnostic of diagnostics){
            if (!ignoredCode.includes(diagnostic.code)) {
                errors.push(new Error(`${diagnostic.file?.fileName}: ${diagnostic.messageText} ts(${diagnostic.code})`));
            }
        }
        if (emitOutput.outputFiles.length) {
            const code = emitOutput.outputFiles[0].text;
            const filename = emitOutput.outputFiles[0].name;
            codeMap.push([
                filename,
                code
            ]);
            try {
                closureMap[filename] = codeToClosure(code, filename);
            } catch (e) {
                e.message = `${filename}: ${e.message}`;
                errors.push(e);
            }
        }
    }
    if (!Object.keys(closureMap).length) {
        errors.push(errNoCode);
        return {
            component: null,
            defaultExport: null,
            compiled: codeMap,
            errors,
            cleanup: noop
        };
    }
    try {
        const parsedEntryFile = normalizePath(entryFile, '/index.js');
        const closure = closureMap[parsedEntryFile];
        if (!closure) {
            errors.push(new Error(`No entry file emitted: '${parsedEntryFile}'.`));
            return {
                component: null,
                defaultExport: null,
                compiled: codeMap,
                errors,
                cleanup: noop
            };
        }
        const mergedResolve = mergeResolve(resolve);
        await loadExternalsToClosureMap(mergedResolve, closureMap);
        const result = closure(createRequireFn(closureMap, requireFn, mergedResolve));
        if (!result.default.displayName) {
            result.default.displayName = displayName;
        }
        return {
            component: React.createElement(result.default),
            defaultExport: result.default,
            compiled: codeMap,
            errors,
            cleanup: ()=>{
                for (const filename of cleanupFiles){
                    const style = document.head.querySelector(`style[data-tsx-browser-compiler-filename="${filename}"]`);
                    style?.remove();
                }
            }
        };
    } catch (e) {
        errors.push(e);
        return {
            component: null,
            defaultExport: null,
            compiled: codeMap,
            errors,
            cleanup: noop
        };
    }
};

export { VERSION, asyncTsxToElement };
